;
;ZProg
;Verion 1.0
;
;Author(s):
;  Marcel Kanter (m_kanter@web.de)
;
;DESCRIPTION
;===========
;
;ZProg is the successor of the AVRISP from Atmel. It understands the
;commands of the original AVRISP, thus it's possible to use the existing
;programs with ZProg.
;
;The new commands give the software developer full control over the
;programmer. So it's possible to support more devices without
;changing/reprogramming the firmware.
;
;The new commands are: 'G' (check for extended functionality), 'H' (extended
;functions), 'I' (bridge command). For details see "Commands" section.
;
;Commands
;--------
;                                    +-------------+------------+------+
;                                    | Host writes | Host reads |      |
;                                    +-------------+------------+      |
;                                    | ID  data    | data       | Note |
;+-----------------------------------+-------------+------------+------+
;| Enter programming mode            | 'P'         | 13d        |   1  |
;| Autoincrement address support?    | 'a'         | 'Y'/'N'    |      |
;| Set address                       | 'A' ah al   | 13d        |   2  |
;| Write program memory, low byte    | 'c' dd      | 13d        |   3  |
;| Write program memory, high byte   | 'C' dd      | 13d        |   3  |
;| Issue Page Write                  | 'm'         | 13d        |  13  |
;| Read program memory               | 'R'         | dd(dd)     |   4  |
;| Write data memory                 | 'D' dd      | 13d        |      |
;| Read data memory                  | 'd'         | dd         |      |
;| Chip erase                        | 'e'         | 13d        |      |
;| Write lock bits                   | 'l' dd      | 13d        |      |
;| Write fuse bits                   | 'f' dd      | 13d        |  11  |
;| Read fuse and lock bits           | 'F'         | dd         |  11  |
;| Leave programming mode            | 'L'         | 13d        |   5  |
;| Select device type                | 'T' dd      | 13d        |   6  |
;| Read signature bytes              | 's'         | 3*dd       |      |
;| Return supported device codes     | 't'         | n*dd  00d  |   7  |
;| Return software identifier        | 'S'         | 7*dd       |   8  |
;| Return sofware version            | 'V'         | dd dd      |   9  |
;| Return hardware version           | 'v'         | dd dd      |   9  |
;| Return programmer type            | 'p'         | dd         |  10  |
;| Set LED                           | 'x' dd      | 13d        |  12  |
;| Clear LED                         | 'y' dd      | 13d        |  12  |
;| Universial command                | ':' 3*dd    | dd  13d    |      |
;| New universal command             | '.' 4*dd    | dd  13d    |      |
;| Special test command              | 'Z' 2*dd    | dd         |      |
;+-----------------------------------+-------------+------------+------+
;| Extended function support?        | 'G'         | 'Y'/'?'    |  14  |
;| Extended functions                | 'H'         | n*dd       |  15  |
;|   Get oscillator frequency        | 'H' 'Q'     | 5*dd       |  15  |
;|   Get SPI bus prescalers          | 'H' 's'     | n*dd 00d   |  15  |
;|   Set SPI bus prescaler index     | 'H' 'S' dd  | 13d        |  15  |
;|   Set Reset Line                  | 'H' 'R'     | 13d        |  15  |
;|   Clear Reset Line                | 'H' 'r'     | 13d        |  15  |
;|   Turn LED on                     | 'H' 'L'     | 13d        |  15  |
;|   Turn LED off                    | 'H' 'l'     | 13d        |  15  |
;| USART SPI bridge                  | 'I' dd      | dd         |  16  |
;+-----------------------------------+-------------+------------+------+
;
;NOTE 1
;  The Enter programming mode command MUST be sent one time prior to
;  the other commands, with the exception of the 't', 'S', 'V', 'v'
;  and 'T' commands. The 'T' command must be sent before this command
;  (see note 6).
;
;  For programmers supporting both parallel and serial programming
;  mode this command enters parallel programming mode. For programmers
;  supporting only serial programming mode, this command enters serial
;  programming mode.
;
;NOTE 2
;  The ah and al are the high and low order bytes of the address. For
;  parallel programmers this command issues the Load Address Low/High
;  Byte command. For serial programmers the address byte is stored for
;  use by the Read/Write commands.
;
;NOTE 3
;  For parallel programmers this command issues the Program Flash
;  command. For serial programmers this command iussues the Write
;  Program Memory Command. For devices with byte-wide program memories
;  only the low byte command should be used.
;
;NOTE 4
;  The contents of the program memory at the address given by the 'A'
;  command are written to the serial port in binary form. For byte
;  wide memories one byte is written. For 16 bit memories two bytes
;  are written, MSB first.
;
;NOTE 5
;  This command must be executed after the programming is finished.
;
;NOTE 6
;  The select device type command must be sent before the enter
;  programming command
;
;NOTE 7
;  The supported device codes are returned in binary form terminated
;  by 0x00.
;
;NOTE 8
;  This return a 7 character ASCII string identifying the programmer.
;  For the development board it is "AVR DEV", for the parallel
;  programmer it is "AVR PPR" and for the in-curcuit programmer it is
;  "AVR ICP".
;  ZProg should return "ZProg".
;
;NOTE 9
;  The software/hardware version are returned as two ASCII numbers.
;
;NOTE 10
;  This command should be used to identify the programmer type. The
;  return value is 'S' for serial (or SPI) programmers or 'P' for
;  parallel programmers.
;
;NOTE 11
;  The write fuse bits command are available only on parallel
;  programmers and only for AVR devices (device code < 0x80). The host
;  should use the return programmer type command to determine the
;  programmer type, do not use the  "AVR PPR" idenifier because other
;  programmers may be available in the future.
;
;NOTE 12
;  Currently only the AVR development board has LEDs. The other boards
;  must implement this commands as NOPs.
;
;NOTE 13
;  Devices using Page Mode Programming write one page of flash memory
;  before issuing a Page Mode Write Pulse.
;
;NOTE 14
;  This command will return an 'Y' if the extended functions are supported.
;  Due to backward compatibily an '?' will be returned if these functions
;  are not supported.
;
;NOTE 15
;  These are the extended functions.
;  Before using this function, check if the programmer support the extended
;  functions (see 'G' command).
;  The first data byte send to the programmer may contain one of the following:
;    'Q'
;      Give back the oscillator frequency in MHz. The programmer returns 5
;      bytes: "xxyyy" where xx stands for the full MHz of the frequency and
;      yyy stands for the kHz part of the frequency. For an oscillator with
;      14.7456 Mhz it would be "14746". This value is needed to calculate
;      the speed of the SPI bus.
;
;    's'
;      Returns the SPI bus frequency prescalers in ascending order, terminated
;      with an zero sign.
;      These values and the oscillator frequency can be used to calculate and
;      set the speed of the SPI bus (see 'S' function).
;      fspi = fosc / prescaler
;      The ascending order (an value may occur twice or more - but in order)
;      is used to simplify the speed selection, an higher
;      prescaler will result in slower SPI bus speed. (see function 'S')
;      The return value "02 04 08 10 20 40 40 80 00" will give the divisors
;      2, 4, 8, 16, 32, 64, 64 and 128.
;
;    'S'
;      Sets an prescaler. Here the index (1 byte) of the prescaler will be
;      send. In the example above the prescaler 16 is selected by sending an
;      '03' to the programmer.
;      When the index is in range, the programmer will return an 13d, if it's
;      not in range an '?' will be returned.
;
;    'R'
;      Sets the reset line of the programmer. This command overwrites the current
;      state of the line. The programmer returns 13d.
;
;    'r'
;      Clears the reset line of the programmer. This command overwrites the
;      current state of the line. The programmer returns 13d.
;
;    'L'
;      Turns the LED of the programmer on. This command overwrites the current
;      state. The programmer returns 13d.
;
;    'l'
;      Turns the LED of the programmer off. This command overwrites the
;      current state. The programmer returns 13d.
;
;NOTE 16
;  This is an pure bridge command. The byte send to the programmer will be
;  transfered through the SPI bus and the result is send back to the PC.
;  Before using this function, check if the programmer support the extended
;  functions (see 'G' command).
;
;Device List    
;-----------
;
;+------------+----------+------+------+------+-------+--------+
;|   Device   |Signature | Code | Lock | Fuse | PMode | Tested |
;+------------+----------+------+------+------+-------+--------+
;|C1051       |          | 0x80 |      |      |       |        |
;|C2051       |          | 0x81 |      |      |       |        |
;|S53         |          | 0x87 |      |      |       |        |
;|S8252       |          | 0x86 |      |      |       |        |
;|            |          |      |      |      |       |        |
;|Mega103     | 1E 97 01 | 0x41 | R/W  | R/W  | Page  |        |
;|Mega128     |          | 0x43 |      |      |       |        |
;|Mega16      | 1E 94 03 | 0x74 |      |      |       |        |
;|Mega161     | 1E 94 01 | 0x60 | R/W  | R/W  | Page  |        |
;|Mega163     | 1E 94 02 | 0x64 | R/W  | R/W  | Page  |        |
;|Mega603     |          | 0x42 |      |      |       |        |
;|Mega64      |          | 0x45 |      |      |       |        |
;|Mega8       | 1E 93 07 | 0x76 |      |      |       |        |
;|Mega83      | 1E 93 05 | 0x65 | R/W  | R/W  | Page  |        |
;|Mega8515    |          | 0x3A |      |      |       |        |
;|            |          |      |      |      |       |        |
;|Tiny12      | 1E 90 05 | 0x55 | R/W  | R/W  | Byte  |        |
;|Tiny15      | 1E 90 06 | 0x56 | R/W  | R/W  | Byte  |        |
;|            |          |      |      |      |       |        |
;|S1200A      | 1E 90 01 | 0x10 |      |      |       |        |
;|S1200B      | 1E 90 01 | 0x11 |      |      |       |        |
;|S1200C      | 1E 90 01 | 0x12 |      |      |       |        |
;|S1200D      | 1E 90 01 | 0x13 |  W   | NA   | Byte  |        |
;|S2313       | 1E 91 01 | 0x20 |  W   | NA   | Byte  |        |
;|S2323       | 1E 91 02 | 0x48 | R/W  | R/W  | Byte  |        |
;|S2333       | 1E 91 05 | 0x34 | R/W  | R/W  | Byte  |        |
;|S2343       | 1E 91 03 | 0x4C | R/W  | R/W  | Byte  |        |
;|S4414       | 1E 92 01 | 0x28 |  W   | NA   | Byte  |        |
;|S4433       | 1E 92 03 | 0x30 | R/W  | R/W  | Byte  |        |
;|S4434       | 1E 92 02 | 0x6C | R/W  | R/W  | Byte  |        |
;|S8515       | 1E 93 01 | 0x38 |  W   | NA   | Byte  |        |
;|S8535       | 1E 93 03 | 0x68 | R/W  | R/W  | Byte  |        |
;+------------+----------+------+------+------+-------+--------+
;
;LEGEND:
;
;Signature - Device Signature Byte
;Code      - Unique device code used by AVRProg to identify device
;Lock      - Lockbits
;Fuse      - Fusebits
;PMode     - Indicates if device uses byte or page programming mode
;Tested    - Indicates if device was tested with this firmware
; 
;R/W - Read and Write Access
;R   - Read Access Only
;W   - Write Access Only
;NA  - Not Accessible
;


;include for the device in the programmer
#include "tn2313def.inc"

;BAUDRATE (to PC)
#define BAUDRATE 7 ;@ 15MHz: 115200k w/ 1.7% error
;#define BAUDRATE 95 ;@ 15MHz: 9600k w/ 1.7% error

;DELAY COUNTER
#define DELAY_COUNT_1mS 5000 ;@ 15MHz: 15000k cycles / 3 cycles per count

;DEVICE CODES
#define C1051 0x80
#define C2051 0x81
#define S53 0x87
#define S8252 0x86
#define Mega103 0x41
#define Mega128 0x43
#define Mega16 0x74
#define Mega161 0x60
#define Mega163 0x64
#define Mega603 0x42
#define Mega64 0x45
#define Mega8 0x76
#define Mega83 0x65
#define Mega8515 0x3A
#define Tiny12 0x55
#define Tiny15 0x56
#define S1200D 0x13
#define S2313 0x20
#define S2323 0x48
#define S2333 0x34
#define S2343 0x4C
#define S4414 0x28
#define S4433 0x30
#define S4434 0x6C
#define S8515 0x38
#define S8535 0x68

;CURRENT VERSION CODES
#define SW_MAJOR '1' ;Major SW revision number
#define SW_MINOR '0' ;Minor SW revision number
#define HW_MAJOR '1' ;Major HW revision number
#define HW_MINOR '0' ;Minor HW revision number


;customize to your hardware
;LED
#define LED_PORT PORTD, 2
#define LED_DDR DDRD, 2
#define LED_PIN PIND, 2 ;not used - a read should give the state of LED_PORT
;RESET PIN
#define RESET_PORT PORTB, 4
#define RESET_DDR DDRB, 4
#define RESET_PIN PINB, 4 ;not used - a read should give the state of RESET_PORT
;SCK
#define SCK_PORT PORTB, 7
#define SCK_DDR DDRB, 7
#define SCK_PIN PINB, 7
;MISO (DI)
#define DI_PORT PORTB, 5
#define DI_DDR DDRB, 5
#define DI_PIN PINB, 5
;MOSI (DO)
#define DO_PORT PORTB, 6
#define DO_DDR DDRB, 6
#define DO_PIN PINB, 6


;REGISTERs
;#define TEMP0 r16 ;temp register for argument passing
#define DEVICE r21 ;the current device code
#define SPI_PRESCALE r22 ;the prescaling factor for the usi-spi

;INTERRUPT VECTORS
rjmp RESET
rjmp EXTINT0
rjmp EXTINT1
rjmp TIM1_CAPT
rjmp TIM1_COMPA
rjmp TIM1_OVF
rjmp TIM0_OVF
rjmp USART0_RXC
rjmp USART0_DRE
rjmp USART0_TXC
rjmp ANA_COMP
rjmp PCINT
rjmp TIMER1_COMPB
rjmp TIMER0_COMPA
rjmp TIMER0_COMPB
rjmp USI_START
rjmp USI_OVERFLOW
rjmp EE_READY
rjmp WDT_OVERFLOW

EXTINT0:
EXTINT1:
TIM1_CAPT:
TIM1_COMPA:
TIM1_OVF:
TIM0_OVF:
USART0_RXC:
USART0_DRE:
USART0_TXC:
ANA_COMP:
PCINT:
TIMER1_COMPB:
TIMER0_COMPA:
TIMER0_COMPB:
USI_START:
USI_OVERFLOW:
EE_READY:
WDT_OVERFLOW:


;RESET
;setup the programmer and then enter the main loop
RESET:
ldi r16, low(RAMEND)
out SPL, r16
sei

;init the programmer
rcall setup

;main loop
main:
;get byte from PC
rcall usart_receive

;ESC - nothing to do
cpi r16, 0x1B
breq main

cpi r16, 'I'
brne _main_write_program_high

  rcall usart_receive
  rcall spi_transfer
  rcall usart_transmit
  rjmp main

;write program memory, high byte
_main_write_program_high:
cpi r16, 'C'
brne _main_write_program_low

;write program memory, low byte
_main_write_program_low:
cpi r16, 'c'
brne _main_read_program

;read program memory
_main_read_program:
cpi r16, 'R'
brne _main_load_address

;load the address
_main_load_address:
cpi r16, 'A'
brne _main_write_data

;write data memory
_main_write_data:
cpi r16, 'D'
brne _main_read_data

;read data memory
_main_read_data:
cpi r16, 'd'
brne _main_write_fuse

;write fuse bits
_main_write_fuse:
cpi r16, 'f'
brne _main_read_fuse

;read fuse and lock bits
_main_read_fuse:
cpi r16, 'F'
brne _main_write_lock

;write lock bits
_main_write_lock:
cpi r16, 'l'
brne _main_write_program_page

;write program memory page
_main_write_program_page:
cpi r16, 'm'
brne _main_universal_command

;universal command
_main_universal_command:
cpi r16, ':'
brne _main_new_universal

;new universal command
_main_new_universal:
cpi r16, '.'
brne _main_enter_program

;enter program mode
_main_enter_program:
cpi r16, 'P'
brne _main_leave_program

  ;check if device set before - check each supported code
  cpi DEVICE, Mega16
  breq _main_enter_program0
  cpi DEVICE, S2323
  breq _main_enter_program0
  cpi DEVICE, S2333
  breq _main_enter_program0
  cpi DEVICE, S2343
  breq _main_enter_program0
  cpi DEVICE, S4414
  breq _main_enter_program0
  cpi DEVICE, S4433
  breq _main_enter_program0
  cpi DEVICE, S4434
  breq _main_enter_program0

  ;an unsupported device set before
  rjmp _main_error

  _main_enter_program0:
  
  ;set the ddrs to correct direction
  rcall set_ddrs

  ;turn led on
  sbi LED_PORT

  _main_enter_program1:

  ;clear SCK
  cbi SCK_PORT

  ;slow high pulse on reset
  ;set RESET (HIGH - no reset)
  sbi RESET_PORT
  ;small delay
  ldi r16, 0xFF
  rcall delay
  ;clear RESET (LOW - reset)
  cbi RESET_PORT
  ;small delay
  ldi r16, 0xFF
  rcall delay

  ;delay 20ms
  ldi r16, 20
  rcall delay_mS

  ;enter program mode
  ldi r16, 0xAC
  rcall spi_transfer
  ldi r16, 0x53
  rcall spi_transfer
  ldi r16, 0x00
  rcall spi_transfer
  mov r17, r16 ;save for sync test
  ldi r16, 0x00
  rcall spi_transfer

  ;sync test
  cpi r17, 0x53
  breq _main_enter_program2

  ;slow down spi
  inc SPI_PRESCALE
  ;try again
  rjmp _main_enter_program1

  _main_enter_program2:

  ;continue with ack
  rjmp _main_transmit_ack

;leave program mode
_main_leave_program:
cpi r16, 'L'
brne _main_chip_erase

  ;set all pins to input
  rcall clear_ddrs

  ;set RESET (HIGH - no reset)
  sbi RESET_PORT
  ;small delay
  ldi r16, 0xFF
  rcall delay

  ;turn led off
  cbi LED_PORT

  ;continue with ack
  rjmp _main_transmit_ack

;chip erase
_main_chip_erase:
cpi r16, 'e'
brne _main_device_type

;device type
_main_device_type:
cpi r16, 'T'
brne _main_ret_sw_ident

  ;read the device code
  rcall usart_receive
  mov DEVICE, r16
  ;continue with ack
  rjmp _main_transmit_ack

;return software identifier
_main_ret_sw_ident:
cpi r16, 'S'
brne _main_ret_sw_ver

  ;send the id
  ldi r16, 'A'
  rcall usart_transmit
  ldi r16, 'V'
  rcall usart_transmit
  ldi r16, 'R'
  rcall usart_transmit
  ldi r16, 'Z'
  rcall usart_transmit
  ldi r16, 'P'
  rcall usart_transmit
  ldi r16, 'r'
  rcall usart_transmit
  ldi r16, 'g'
  rcall usart_transmit
  ;continue
  rjmp main

;return software version
_main_ret_sw_ver:
cpi r16, 'V'
brne _main_ret_hw_ver

  ;send sw version characters
  ldi r16, SW_MAJOR
  rcall usart_transmit
  ldi r16, SW_MINOR
  rcall usart_transmit
  ;continue
  rjmp main

;return hardware version
_main_ret_hw_ver:
cpi r16, 'v'
brne _main_supported_devices

  ;send hw version characters
  ldi r16, HW_MAJOR
  rcall usart_transmit
  ldi r16, HW_MINOR
  rcall usart_transmit
  ;continue
  rjmp main

;show supported devices
_main_supported_devices:
cpi r16, 't'
brne _main_programmer_type

  ;send an list of all supported device codes - see table above
  ldi r16, Mega16
  rcall usart_transmit
  ldi r16, S2323
  rcall usart_transmit
  ldi r16, S2333
  rcall usart_transmit
  ldi r16, S2343
  rcall usart_transmit
  ldi r16, S4414
  rcall usart_transmit
  ldi r16, S4433
  rcall usart_transmit
  ldi r16, S4434
  rcall usart_transmit

  ldi r16, 0x00 ;end of list
  rcall usart_transmit
  ;continue
  rjmp main

;return programmer type
_main_programmer_type:
cpi r16, 'p'
brne _main_read_signature

  ;send 'S' (for serial programmer)
  ldi r16, 'S'
  rcall usart_transmit
  ;continue
  rjmp main

;read signature bytes
_main_read_signature:
cpi r16, 's'
brne _main_auto_inc_addr

  ;transfer the 3 signature bytes
  ldi r17, 0x02
  rcall _main_read_signature0
  ldi r17, 0x01
  rcall _main_read_signature0
  ldi r17, 0x00
  rcall _main_read_signature0
  ;continue
  rjmp main

  ;helper routine
  _main_read_signature0:
  ;transfer byte 1
  ldi r16, 0x30
  rcall spi_transfer
  ;transfer byte 2
  ldi r16, 0x00
  rcall spi_transfer
  ;transfer byte 3 (specific)
  mov r16, r17
  rcall spi_transfer
  ;transfer dummy data
  ldi r16, 0x00
  rcall spi_transfer
  ;send the received data to PC
  rcall usart_transmit
  ret

;return autoincrement address
_main_auto_inc_addr:
cpi r16, 'a'
brne _main_set_led

  ;send 'Y' (for support)
  ldi r16, 'Y'
  rcall usart_transmit
  ;continue
  rjmp main

;set led
_main_set_led:
cpi r16, 'x'
brne _main_clear_led

  ;read data - do nothing with it
  rcall usart_receive
  ;contiunue with ack
  rjmp _main_transmit_ack

;clear led
_main_clear_led:
cpi r16, 'y'
brne _main_error

  ;read data - do nothing with it
  rcall usart_receive
  ;contiunue with ack
  rjmp _main_transmit_ack

;an error occoured
_main_error:
ldi r16, '?'
rcall usart_transmit

  ;continue
  rjmp main

;transmit an ack/ok
_main_transmit_ack:
ldi r16, 13
rcall usart_transmit

  ;continue
  rjmp main

;end of main


;initializes the registers/data,
;led and reset output, usart, usi-spi
setup:
;no device selected
ldi r16, 0xFF
mov DEVICE, r16

;fastest spi mode
ldi r16, 0
mov SPI_PRESCALE, r16

;led: off
cbi LED_PORT
;led: output
sbi LED_DDR

;reset: high (no reset)
sbi RESET_PORT
;reset: output
sbi RESET_DDR

;setup the usart
;ucsrb: no interrupts, receiver + transmitter enable, 8 bit data
ldi r16, 0b00011000
out UCSRB, r16
;ucsrc: async. mode, no parity, 1 stop bit, 8 bit data
ldi r16, 0b00000110
out UCSRC, r16
;baud rate (as selected above)
ldi r16, high(BAUDRATE)
out UBRRH, r16
ldi r16, low(BAUDRATE)
out UBRRL, r16

;setup the usi-spi
;sck: low
cbi SCK_PORT
;miso (DI): pull-up off
cbi DI_PORT
;mosi (DO): low
cbi DO_PORT
;I/O pins as input
rcall clear_ddrs
;usicr: no interrupts, 3 wire mode, sw toggle
ldi r16, 0b00011010
out USICR, r16
ret


;all I/O pins to correct direction
set_ddrs:
sbi SCK_DDR
cbi DI_DDR
sbi DO_DDR
ret


;all I/O pins to input
clear_ddrs:
cbi SCK_DDR
cbi DI_DDR ;already input
cbi DO_DDR
ret


;receives an byte from PC (to r16)
;waits until the usart hw received a byte
usart_receive:
sbis UCSRA, RXC
rjmp usart_receive
in r16, UDR
ret


;sends an byte (from r16) to PC
;waits until the usart hw is ready to send (buffer clear)
usart_transmit:
sbis UCSRA, UDRE
rjmp usart_transmit
out UDR, r16
ret


;transfers an byte via USI-SPI
;the data to send and the received data is in r16
spi_transfer:
push r17
out USIDR, r16
;clear overflow flag and reset counter
ldi r16, 1<<USIOIF
out USISR, r16
;set the toggle bit - no change in configuration
in r16, USICR
ori r16, 1<<USITC

;unrolled loop (+ 80 byte): 16 out commands for each flank one
mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer0:
subi r17, 1
brsh spi_transfer0

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer1:
subi r17, 1
brsh spi_transfer1

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer2:
subi r17, 1
brsh spi_transfer2

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer3:
subi r17, 1
brsh spi_transfer3

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer4:
subi r17, 1
brsh spi_transfer4

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer5:
subi r17, 1
brsh spi_transfer5

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer6:
subi r17, 1
brsh spi_transfer6

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer7:
subi r17, 1
brsh spi_transfer7

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer8:
subi r17, 1
brsh spi_transfer8

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer9:
subi r17, 1
brsh spi_transfer9

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer10:
subi r17, 1
brsh spi_transfer10

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer11:
subi r17, 1
brsh spi_transfer11

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer12:
subi r17, 1
brsh spi_transfer12

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer13:
subi r17, 1
brsh spi_transfer13

mov r17, SPI_PRESCALE
out USICR, r16
spi_transfer14:
subi r17, 1
brsh spi_transfer14

mov r17, SPI_PRESCALE
out USICR, r16
;spi_transfer15:
;subi r17, 1
;brsh spi_transfer15

;read the received value
in r16, USIDR
pop r17
ret


;delay function
;wastes a defined time (value: r16, in mS)
delay_ms:
cpi r16, 0
breq _delay_ms_end

;waste 1mS
ldi r17, high(DELAY_COUNT_1mS)
_delay_ms0:
  ldi r18, low(DELAY_COUNT_1mS)
  _delay_ms1:
    dec r18
  brne _delay_ms1
  dec r17
brne _delay_ms0

dec r16
rjmp delay_ms

_delay_ms_end:
ret


;delay function for small delay
;wastes a defined time (value: r16, in count cycles (clock / 3))
delay:
dec r16
brne delay
ret
